@@ -87,7 +87,7 @@ import org.slf4j.LoggerFactory;
 /**
  * Implements a bookie.
  */
-public class BookieImpl extends BookieCriticalThread implements Bookie {
+public class BookieImpl implements Bookie {
 
     private static final Logger LOG = LoggerFactory.getLogger(Bookie.class);
 
@@ -119,6 +119,8 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
 
     protected StateManager stateManager;
 
+    private BookieCriticalThread bookieThread;
+
     // Expose Stats
     final StatsLogger statsLogger;
     private final BookieStats bookieStats;
@@ -390,7 +392,6 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
                       ByteBufAllocator allocator,
                       Supplier<BookieServiceInfo> bookieServiceInfoProvider)
             throws IOException, InterruptedException, BookieException {
-        super("Bookie-" + conf.getBookiePort());
         this.bookieServiceInfoProvider = bookieServiceInfoProvider;
         this.statsLogger = statsLogger;
         this.conf = conf;
@@ -433,7 +434,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         // instantiate the journals
         journals = Lists.newArrayList();
         for (int i = 0; i < journalDirectories.size(); i++) {
-            journals.add(new Journal(i, journalDirectories.get(i),
+            journals.add(Journal.newJournal(i, journalDirectories.get(i),
                     conf, ledgerDirsManager, statsLogger.scope(JOURNAL_SCOPE), allocator, journalAliveListener));
         }
 
@@ -496,6 +497,21 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         this.bookieStats = new BookieStats(statsLogger, journalDirectories.size(), conf.getJournalQueueSize());
     }
 
+    @VisibleForTesting
+    public static BookieImpl newBookieImpl(ServerConfiguration conf,
+                                           RegistrationManager registrationManager,
+                                           LedgerStorage storage,
+                                           DiskChecker diskChecker,
+                                           LedgerDirsManager ledgerDirsManager,
+                                           LedgerDirsManager indexDirsManager,
+                                           StatsLogger statsLogger,
+                                           ByteBufAllocator allocator,
+                                           Supplier<BookieServiceInfo> bookieServiceInfoProvider)
+            throws IOException, InterruptedException, BookieException {
+        return new BookieImpl(conf, registrationManager, storage, diskChecker,
+                ledgerDirsManager, indexDirsManager, statsLogger, allocator, bookieServiceInfoProvider);
+    }
+
     StateManager initializeStateManager() throws IOException {
         return new BookieStateManager(conf, statsLogger, registrationManager,
                 ledgerDirsManager, bookieServiceInfoProvider);
@@ -527,7 +543,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
                             masterKeyCache.put(ledgerId, masterKey);
 
                             // Force to re-insert the master key in ledger storage
-                            handles.getHandle(ledgerId, masterKey, true);
+                            handles.getHandle(ledgerId, masterKey);
                         } else {
                             throw new IOException("Invalid journal. Contains journalKey "
                                     + " but layout version (" + journalVersion
@@ -539,7 +555,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
                             if (key == null) {
                                 key = ledgerStorage.readMasterKey(ledgerId);
                             }
-                            LedgerDescriptor handle = handles.getHandle(ledgerId, key, true);
+                            LedgerDescriptor handle = handles.getHandle(ledgerId, key);
                             handle.setFenced();
                         } else {
                             throw new IOException("Invalid journal. Contains fenceKey "
@@ -557,7 +573,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
                             if (key == null) {
                                 key = ledgerStorage.readMasterKey(ledgerId);
                             }
-                            LedgerDescriptor handle = handles.getHandle(ledgerId, key, true);
+                            LedgerDescriptor handle = handles.getHandle(ledgerId, key);
                             handle.setExplicitLac(explicitLacBuf);
                         } else {
                             throw new IOException("Invalid journal. Contains explicitLAC " + " but layout version ("
@@ -580,7 +596,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
                         if (key == null) {
                             key = ledgerStorage.readMasterKey(ledgerId);
                         }
-                        LedgerDescriptor handle = handles.getHandle(ledgerId, key, true);
+                        LedgerDescriptor handle = handles.getHandle(ledgerId, key);
 
                         recBuff.rewind();
                         handle.addEntry(Unpooled.wrappedBuffer(recBuff));
@@ -641,8 +657,10 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
 
     @Override
     public synchronized void start() {
-        setDaemon(true);
-        ThreadRegistry.register("BookieThread", true);
+        bookieThread = new BookieCriticalThread(() -> run(), "Bookie-" + conf.getBookiePort());
+        bookieThread.setDaemon(true);
+
+        ThreadRegistry.register("BookieThread", 0);
         if (LOG.isDebugEnabled()) {
             LOG.debug("I'm starting a bookie with journal directories {}",
                     journalDirectories.stream().map(File::getName).collect(Collectors.joining(", ")));
@@ -702,7 +720,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         syncThread.start();
 
         // start bookie thread
-        super.start();
+        bookieThread.start();
 
         // After successful bookie startup, register listener for disk
         // error/full notifications.
@@ -726,6 +744,20 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         }
     }
 
+    @Override
+    public void join() throws InterruptedException {
+        if (bookieThread != null) {
+            bookieThread.join();
+        }
+    }
+
+    public boolean isAlive() {
+        if (bookieThread == null) {
+            return false;
+        }
+        return bookieThread.isAlive();
+    }
+
     /*
      * Get the DiskFailure listener for the bookie
      */
@@ -809,7 +841,6 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         return stateManager.isRunning();
     }
 
-    @Override
     public void run() {
         // start journals
         for (Journal journal: journals) {
@@ -902,7 +933,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
             throws IOException, BookieException {
         final long ledgerId = entry.getLong(entry.readerIndex());
 
-        return handles.getHandle(ledgerId, masterKey, false);
+        return handles.getHandle(ledgerId, masterKey);
     }
 
     private Journal getJournal(long ledgerId) {
@@ -1011,7 +1042,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
         ByteBuf explicitLACEntry = null;
         try {
             long ledgerId = entry.getLong(entry.readerIndex());
-            LedgerDescriptor handle = handles.getHandle(ledgerId, masterKey, false);
+            LedgerDescriptor handle = handles.getHandle(ledgerId, masterKey);
             synchronized (handle) {
                 entry.markReaderIndex();
                 handle.setExplicitLac(entry);
@@ -1100,7 +1131,7 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
      */
     public CompletableFuture<Boolean> fenceLedger(long ledgerId, byte[] masterKey)
             throws IOException, BookieException {
-        LedgerDescriptor handle = handles.getHandle(ledgerId, masterKey, false);
+        LedgerDescriptor handle = handles.getHandle(ledgerId, masterKey);
         return handle.fenceAndLogInJournal(getJournal(ledgerId));
     }
 
