@@ -42,6 +42,7 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.TimeUnit;
+import java.util.function.Consumer;
 import org.apache.bookkeeper.bookie.LedgerDirsManager.NoWritableLedgerDirException;
 import org.apache.bookkeeper.bookie.stats.JournalStats;
 import org.apache.bookkeeper.common.collections.BatchedArrayBlockingQueue;
@@ -66,13 +67,15 @@ import org.slf4j.LoggerFactory;
 /**
  * Provide journal related management.
  */
-public class Journal extends BookieCriticalThread implements CheckpointSource {
+public class Journal implements CheckpointSource {
 
     private static final Logger LOG = LoggerFactory.getLogger(Journal.class);
 
     private static final RecyclableArrayList.Recycler<QueueEntry> entryListRecycler =
         new RecyclableArrayList.Recycler<QueueEntry>();
 
+    private BookieCriticalThread thread;
+
     /**
      * Filter to pickup journals.
      */
@@ -461,16 +464,17 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         volatile boolean running = true;
         // This holds the queue entries that should be notified after a
         // successful force write
-        Thread threadToNotifyOnEx;
+        Consumer<Void> threadToNotifyOnEx;
 
         // should we group force writes
         private final boolean enableGroupForceWrites;
         private final Counter forceWriteThreadTime;
 
-        public ForceWriteThread(Thread threadToNotifyOnEx,
+        public ForceWriteThread(Consumer<Void> threadToNotifyOnEx,
                                 boolean enableGroupForceWrites,
                                 StatsLogger statsLogger) {
             super("ForceWriteThread");
+            this.setPriority(Thread.MAX_PRIORITY);
             this.threadToNotifyOnEx = threadToNotifyOnEx;
             this.enableGroupForceWrites = enableGroupForceWrites;
             this.forceWriteThreadTime = statsLogger.getThreadScopedCounter("force-write-thread-time");
@@ -478,7 +482,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         @Override
         public void run() {
             LOG.info("ForceWrite Thread started");
-            ThreadRegistry.register(super.getName());
+            ThreadRegistry.register(super.getName(), 0);
 
             if (conf.isBusyWaitEnabled()) {
                 try {
@@ -530,7 +534,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             // Regardless of what caused us to exit, we should notify the
             // the parent thread as it should either exit or be in the process
             // of exiting else we will have write requests hang
-            threadToNotifyOnEx.interrupt();
+            threadToNotifyOnEx.accept(null);
         }
 
         private void syncJournal(ForceWriteRequest lastRequest) throws IOException {
@@ -629,18 +633,20 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
     // journal entry queue to commit
     final BatchedBlockingQueue<QueueEntry> queue;
-    final BatchedBlockingQueue<ForceWriteRequest> forceWriteRequests;
+    BatchedBlockingQueue<ForceWriteRequest> forceWriteRequests;
 
     volatile boolean running = true;
     private final LedgerDirsManager ledgerDirsManager;
     private final ByteBufAllocator allocator;
-    private final MemoryLimitController memoryLimitController;
 
     // Expose Stats
     private final JournalStats journalStats;
 
     private JournalAliveListener journalAliveListener;
 
+    private MemoryLimitController memoryLimitController;
+
+
     public Journal(int journalIndex, File journalDirectory, ServerConfiguration conf,
             LedgerDirsManager ledgerDirsManager) {
         this(journalIndex, journalDirectory, conf, ledgerDirsManager, NullStatsLogger.INSTANCE,
@@ -649,7 +655,6 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
 
     public Journal(int journalIndex, File journalDirectory, ServerConfiguration conf,
             LedgerDirsManager ledgerDirsManager, StatsLogger statsLogger, ByteBufAllocator allocator) {
-        super(journalThreadName + "-" + conf.getBookiePort());
         this.allocator = allocator;
 
         StatsLogger journalStatsLogger = statsLogger.scopeLabel("journalIndex", String.valueOf(journalIndex));
@@ -674,8 +679,8 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         this.journalWriteBufferSize = conf.getJournalWriteBufferSizeKB() * KB;
         this.syncData = conf.getJournalSyncData();
         this.maxBackupJournals = conf.getMaxBackupJournals();
-        this.forceWriteThread = new ForceWriteThread(this, conf.getJournalAdaptiveGroupWrites(),
-                journalStatsLogger);
+        this.forceWriteThread = new ForceWriteThread((__) -> this.interruptThread(),
+                conf.getJournalAdaptiveGroupWrites(), journalStatsLogger);
         this.maxGroupWaitInNanos = TimeUnit.MILLISECONDS.toNanos(conf.getJournalMaxGroupWaitMSec());
         this.bufferedWritesThreshold = conf.getJournalBufferedWritesThreshold();
         this.bufferedEntriesThreshold = conf.getJournalBufferedEntriesThreshold();
@@ -722,6 +727,14 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         this.journalAliveListener = journalAliveListener;
     }
 
+    @VisibleForTesting
+    static Journal newJournal(int journalIndex, File journalDirectory, ServerConfiguration conf,
+                                     LedgerDirsManager ledgerDirsManager, StatsLogger statsLogger,
+                                     ByteBufAllocator allocator, JournalAliveListener journalAliveListener) {
+        return new Journal(journalIndex, journalDirectory, conf, ledgerDirsManager, statsLogger, allocator,
+                journalAliveListener);
+    }
+
     JournalStats getJournalStats() {
         return this.journalStats;
     }
@@ -921,6 +934,14 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
         return queue.size();
     }
 
+    @VisibleForTesting
+    JournalChannel newLogFile(long logId, Long replaceLogId) throws IOException {
+        return new JournalChannel(journalDirectory, logId, journalPreAllocSize, journalWriteBufferSize,
+                journalAlignmentSize, removePagesFromCache,
+                journalFormatVersionToWrite, getBufferedChannelBuilder(),
+                conf, fileChannelProvider, replaceLogId);
+    }
+
     /**
      * A thread used for persisting journal entries to journal files.
      *
@@ -936,10 +957,9 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
      * </p>
      * @see org.apache.bookkeeper.bookie.SyncThread
      */
-    @Override
     public void run() {
         LOG.info("Starting journal on {}", journalDirectory);
-        ThreadRegistry.register(journalThreadName);
+        ThreadRegistry.register(journalThreadName, 0);
 
         if (conf.isBusyWaitEnabled()) {
             try {
@@ -989,10 +1009,7 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
                         ? journalIds.get(0) : null;
 
                     journalCreationWatcher.reset().start();
-                    logFile = new JournalChannel(journalDirectory, logId, journalPreAllocSize, journalWriteBufferSize,
-                                        journalAlignmentSize, removePagesFromCache,
-                                        journalFormatVersionToWrite, getBufferedChannelBuilder(),
-                                        conf, fileChannelProvider, replaceLogId);
+                    logFile = newLogFile(logId, replaceLogId);
 
                     journalStats.getJournalCreationStats().registerSuccessfulEvent(
                             journalCreationWatcher.stop().elapsed(TimeUnit.NANOSECONDS), TimeUnit.NANOSECONDS);
@@ -1238,8 +1255,8 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
             forceWriteThread.shutdown();
 
             running = false;
-            this.interrupt();
-            this.join();
+            this.interruptThread();
+            this.joinThread();
             LOG.info("Finished Shutting down Journal thread");
         } catch (IOException | InterruptedException ie) {
             Thread.currentThread().interrupt();
@@ -1267,10 +1284,34 @@ public class Journal extends BookieCriticalThread implements CheckpointSource {
      */
     @VisibleForTesting
     public void joinThread() throws InterruptedException {
-        join();
+        if (thread != null) {
+            thread.join();
+        }
+    }
+
+    public void interruptThread() {
+        if (thread != null) {
+            thread.interrupt();
+        }
+    }
+
+    public synchronized void start() {
+        thread = new BookieCriticalThread(() -> run(), journalThreadName + "-" + conf.getBookiePort());
+        thread.setPriority(Thread.MAX_PRIORITY);
+        thread.start();
     }
 
     long getMemoryUsage() {
         return memoryLimitController.currentUsage();
     }
+
+    @VisibleForTesting
+    void setMemoryLimitController(MemoryLimitController memoryLimitController) {
+        this.memoryLimitController = memoryLimitController;
+    }
+
+    @VisibleForTesting
+    public void setForceWriteRequests(BatchedBlockingQueue<ForceWriteRequest> forceWriteRequests) {
+        this.forceWriteRequests = forceWriteRequests;
+    }
 }
