@@ -25,11 +25,11 @@ import static org.apache.bookkeeper.bookie.BookKeeperServerStats.ACTIVE_LEDGER_C
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.BOOKIE_SCOPE;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.CATEGORY_SERVER;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.DELETED_LEDGER_COUNT;
-import static org.apache.bookkeeper.bookie.BookKeeperServerStats.ENTRY_LOCATION_COMPACTION_COUNT;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.MAJOR_COMPACTION_COUNT;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.MINOR_COMPACTION_COUNT;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.RECLAIMED_COMPACTION_SPACE_BYTES;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.RECLAIMED_DELETION_SPACE_BYTES;
+import static org.apache.bookkeeper.bookie.BookKeeperServerStats.RECLAIM_FAILED_TO_DELETE;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.THREAD_RUNTIME;
 
 import java.util.function.Supplier;
@@ -62,11 +62,6 @@ public class GarbageCollectorStats {
         help = "Number of major compactions"
     )
     private final Counter majorCompactionCounter;
-    @StatsDoc(
-            name = ENTRY_LOCATION_COMPACTION_COUNT,
-            help = "Number of entry location compactions"
-    )
-    private final Counter entryLocationCompactionCounter;
     @StatsDoc(
         name = RECLAIMED_DELETION_SPACE_BYTES,
         help = "Number of disk space bytes reclaimed via deleting entry log files"
@@ -77,6 +72,11 @@ public class GarbageCollectorStats {
         help = "Number of disk space bytes reclaimed via compacting entry log files"
     )
     private final Counter reclaimedSpaceViaCompaction;
+    @StatsDoc(
+            name = RECLAIM_FAILED_TO_DELETE,
+            help = "Number of reclaim failed counts when deleting entry log files"
+    )
+    private final Counter reclaimFailedToDelete;
     @StatsDoc(
         name = DELETED_LEDGER_COUNT,
         help = "Number of ledgers deleted by garbage collection"
@@ -111,9 +111,9 @@ public class GarbageCollectorStats {
 
         this.minorCompactionCounter = statsLogger.getCounter(MINOR_COMPACTION_COUNT);
         this.majorCompactionCounter = statsLogger.getCounter(MAJOR_COMPACTION_COUNT);
-        this.entryLocationCompactionCounter = statsLogger.getCounter(ENTRY_LOCATION_COMPACTION_COUNT);
         this.reclaimedSpaceViaCompaction = statsLogger.getCounter(RECLAIMED_COMPACTION_SPACE_BYTES);
         this.reclaimedSpaceViaDeletes = statsLogger.getCounter(RECLAIMED_DELETION_SPACE_BYTES);
+        this.reclaimFailedToDelete = statsLogger.getCounter(RECLAIM_FAILED_TO_DELETE);
         this.gcThreadRuntime = statsLogger.getOpStatsLogger(THREAD_RUNTIME);
         this.deletedLedgerCounter = statsLogger.getCounter(DELETED_LEDGER_COUNT);
 
