@@ -160,6 +160,9 @@ public class LedgerMetadataIndex implements Closeable {
 
     public Iterable<Long> getActiveLedgersInRange(final long firstLedgerId, final long lastLedgerId)
             throws IOException {
+        if (firstLedgerId <= 0 && lastLedgerId == Long.MAX_VALUE) {
+            return ledgers.keys();
+        }
         return Iterables.filter(ledgers.keys(), new Predicate<Long>() {
             @Override
             public boolean apply(Long ledgerId) {
@@ -336,7 +339,6 @@ public class LedgerMetadataIndex implements Closeable {
                 key.set(ledgerId);
                 ledgersDb.delete(key.array);
                 ++deletedLedgers;
-                pendingDeletedLedgers.remove(ledgerId);
             }
 
             if (log.isDebugEnabled()) {
@@ -350,7 +352,7 @@ public class LedgerMetadataIndex implements Closeable {
     }
 
     private ReentrantLock lockForLedger(long ledgerId) {
-        return locks[Math.abs((int) ledgerId) % locks.length];
+        return locks[(int) (Math.abs(ledgerId) % locks.length)];
     }
 
     int getStorageStateFlags() throws IOException {
