@@ -117,12 +117,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     private static String dbStoragerExecutorName = "db-storage";
     private final ExecutorService executor = Executors.newSingleThreadExecutor(
-            new DefaultThreadFactory(dbStoragerExecutorName) {
-                @Override
-                protected Thread newThread(Runnable r, String name) {
-                    return super.newThread(ThreadRegistry.registerThread(r, dbStoragerExecutorName), name);
-                }
-            });
+            new DefaultThreadFactory(dbStoragerExecutorName));
 
     // Executor used to for db index cleanup
     private final ScheduledExecutorService cleanupExecutor = Executors
@@ -223,6 +218,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         flushExecutorTime = ledgerIndexDirStatsLogger.getThreadScopedCounter("db-storage-thread-time");
 
         executor.submit(() -> {
+            ThreadRegistry.register(dbStoragerExecutorName, 0);
             // ensure the metric gets registered on start-up as this thread only executes
             // when the write cache is full which may not happen or not for a long time
             flushExecutorTime.addLatency(0, TimeUnit.NANOSECONDS);
@@ -313,8 +309,6 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
     public void entryLocationCompact() {
         if (entryLocationIndex.isCompacting()) {
             // RocksDB already running compact.
-            log.info("Compacting directory {}, skipping this entryLocationCompaction this time.",
-                    entryLocationIndex.getEntryLocationDBPath());
             return;
         }
         cleanupExecutor.execute(() -> {
