@@ -22,7 +22,6 @@ package org.apache.bookkeeper.proto;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.protobuf.ExtensionRegistry;
 import io.netty.buffer.ByteBuf;
@@ -55,6 +54,7 @@ import org.apache.bookkeeper.common.util.OrderedExecutor;
 import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.net.BookieSocketAddress;
+import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.BatchedReadEntryCallback;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.ForceLedgerCallback;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.FutureGetListOfEntriesOfLedger;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.GenericCallback;
@@ -261,20 +261,18 @@ public class BookieClientImpl implements BookieClient, PerChannelBookieClientFac
 
         toSend.retain();
         client.obtain((rc, pcbc) -> {
-            try {
-                if (rc != BKException.Code.OK) {
-                    try {
-                        executor.executeOrdered(ledgerId,
-                                () -> cb.writeLacComplete(rc, ledgerId, addr, ctx));
-                    } catch (RejectedExecutionException re) {
-                        cb.writeLacComplete(getRc(BKException.Code.InterruptedException), ledgerId, addr, ctx);
-                    }
-                } else {
-                    pcbc.writeLac(ledgerId, masterKey, lac, toSend, cb, ctx);
+            if (rc != BKException.Code.OK) {
+                try {
+                    executor.executeOrdered(ledgerId,
+                            () -> cb.writeLacComplete(rc, ledgerId, addr, ctx));
+                } catch (RejectedExecutionException re) {
+                    cb.writeLacComplete(getRc(BKException.Code.InterruptedException), ledgerId, addr, ctx);
                 }
-            } finally {
-                ReferenceCountUtil.release(toSend);
+            } else {
+                pcbc.writeLac(ledgerId, masterKey, lac, toSend, cb, ctx);
             }
+
+            ReferenceCountUtil.release(toSend);
         }, ledgerId, useV3Enforced);
     }
 
@@ -356,9 +354,21 @@ public class BookieClientImpl implements BookieClient, PerChannelBookieClientFac
         }
     }
 
-    // Without test, this class should be modifier with "private".
-    @VisibleForTesting
-    static class ChannelReadyForAddEntryCallback
+    private void completeBatchRead(final int rc,
+            final long ledgerId,
+            final long startEntryId,
+            final ByteBufList bufList,
+            final BatchedReadEntryCallback cb,
+            final Object ctx) {
+        try {
+            executor.executeOrdered(ledgerId, () -> cb.readEntriesComplete(rc, ledgerId, startEntryId, bufList, ctx));
+        } catch (RejectedExecutionException ree) {
+            cb.readEntriesComplete(getRc(BKException.Code.InterruptedException),
+                    ledgerId, startEntryId, bufList, ctx);
+        }
+    }
+
+    private static class ChannelReadyForAddEntryCallback
         implements GenericCallback<PerChannelBookieClient> {
         private final Handle<ChannelReadyForAddEntryCallback> recyclerHandle;
 
@@ -368,9 +378,7 @@ public class BookieClientImpl implements BookieClient, PerChannelBookieClientFac
         private long entryId;
         private BookieId addr;
         private Object ctx;
-        // Without test, this class should be modifier with "private".
-        @VisibleForTesting
-        WriteCallback cb;
+        private WriteCallback cb;
         private int options;
         private byte[] masterKey;
         private boolean allowFastFail;
@@ -400,23 +408,14 @@ public class BookieClientImpl implements BookieClient, PerChannelBookieClientFac
         public void operationComplete(final int rc,
                                       PerChannelBookieClient pcbc) {
             if (rc != BKException.Code.OK) {
-                bookieClient.executor.executeOrdered(ledgerId, () -> {
-                    try {
-                        bookieClient.completeAdd(rc, ledgerId, entryId, addr, cb, ctx);
-                    } finally {
-                        ReferenceCountUtil.release(toSend);
-                    }
-                    recycle();
-                });
+                bookieClient.completeAdd(rc, ledgerId, entryId, addr, cb, ctx);
             } else {
-                try {
-                    pcbc.addEntry(ledgerId, masterKey, entryId,
-                            toSend, cb, ctx, options, allowFastFail, writeFlags);
-                } finally {
-                    ReferenceCountUtil.release(toSend);
-                }
-                recycle();
+                pcbc.addEntry(ledgerId, masterKey, entryId,
+                              toSend, cb, ctx, options, allowFastFail, writeFlags);
             }
+
+            ReferenceCountUtil.release(toSend);
+            recycle();
         }
 
         private ChannelReadyForAddEntryCallback(
@@ -505,6 +504,26 @@ public class BookieClientImpl implements BookieClient, PerChannelBookieClientFac
         }, ledgerId);
     }
 
+    @Override
+    public void batchReadEntries(final BookieId address, final long ledgerId, final long startEntryId,
+            final int maxCount, final long maxSize, final BatchedReadEntryCallback cb, final Object ctx,
+            final int flags, final byte[] masterKey, final boolean allowFastFail) {
+        final PerChannelBookieClientPool client = lookupClient(address);
+        if (client == null) {
+            cb.readEntriesComplete(getRc(BKException.Code.BookieHandleNotAvailableException),
+                    ledgerId, startEntryId, null, ctx);
+            return;
+        }
+
+        client.obtain((rc, pcbc) -> {
+            if (rc != BKException.Code.OK) {
+                completeBatchRead(rc, ledgerId, startEntryId, null, cb, ctx);
+            } else {
+                pcbc.batchReadEntries(ledgerId, startEntryId, maxCount, maxSize, cb, ctx, flags, masterKey,
+                        allowFastFail);
+            }
+        }, ledgerId);
+    }
 
     @Override
     public void readEntryWaitForLACUpdate(final BookieId addr,
