@@ -66,7 +66,6 @@ import org.slf4j.MDC;
 public class BookieRequestProcessor implements RequestProcessor {
 
     private static final Logger LOG = LoggerFactory.getLogger(BookieRequestProcessor.class);
-    public static final String TLS_HANDLER_NAME = "tls";
 
     /**
      * The server configuration. We use this for getting the number of add and read
@@ -381,6 +380,10 @@ public class BookieRequestProcessor implements RequestProcessor {
                     checkArgument(r instanceof BookieProtocol.ReadRequest);
                     processReadRequest((BookieProtocol.ReadRequest) r, requestHandler);
                     break;
+                case BookieProtocol.BATCH_READ_ENTRY:
+                    checkArgument(r instanceof BookieProtocol.BatchedReadRequest);
+                    processReadRequest((BookieProtocol.BatchedReadRequest) r, requestHandler);
+                    break;
                 case BookieProtocol.AUTH:
                     LOG.info("Ignoring auth operation from client {}",
                             requestHandler.ctx().channel().remoteAddress());
@@ -577,15 +580,9 @@ public class BookieRequestProcessor implements RequestProcessor {
             response.setStatus(BookkeeperProtocol.StatusCode.EBADREQ);
             writeAndFlush(c, response.build());
         } else {
-            LOG.info("Starting TLS handshake with client on channel {}", c);
             // there is no need to execute in a different thread as this operation is light
             SslHandler sslHandler = shFactory.newTLSHandler();
-            if (c.pipeline().names().contains(BookieNettyServer.CONSOLIDATION_HANDLER_NAME)) {
-                c.pipeline().addAfter(BookieNettyServer.CONSOLIDATION_HANDLER_NAME, TLS_HANDLER_NAME, sslHandler);
-            } else {
-                // local transport doesn't contain FlushConsolidationHandler
-                c.pipeline().addFirst(TLS_HANDLER_NAME, sslHandler);
-            }
+            c.pipeline().addFirst("tls", sslHandler);
 
             response.setStatus(BookkeeperProtocol.StatusCode.EOK);
             BookkeeperProtocol.StartTLSResponse.Builder builder = BookkeeperProtocol.StartTLSResponse.newBuilder();
@@ -684,8 +681,11 @@ public class BookieRequestProcessor implements RequestProcessor {
     private void processReadRequest(final BookieProtocol.ReadRequest r, final BookieRequestHandler requestHandler) {
         ExecutorService fenceThreadPool =
                 null == highPriorityThreadPool ? null : highPriorityThreadPool.chooseThread(requestHandler.ctx());
-        ReadEntryProcessor read = ReadEntryProcessor.create(r, requestHandler,
-                this, fenceThreadPool, throttleReadResponses);
+        ReadEntryProcessor read = r instanceof BookieProtocol.BatchedReadRequest
+                ? BatchedReadEntryProcessor.create((BookieProtocol.BatchedReadRequest) r, requestHandler,
+                this, fenceThreadPool, throttleReadResponses, serverCfg.getMaxBatchReadSize())
+                : ReadEntryProcessor.create(r, requestHandler,
+                        this, fenceThreadPool, throttleReadResponses);
 
         // If it's a high priority read (fencing or as part of recovery process), we want to make sure it
         // gets executed as fast as possible, so bypass the normal readThreadPool
