@@ -122,7 +122,7 @@ public class ByteBufList extends AbstractReferenceCounted {
         return buf;
     }
 
-    private static ByteBufList get() {
+    public static ByteBufList get() {
         ByteBufList buf = RECYCLER.get();
         buf.setRefCnt(1);
         return buf;
@@ -286,29 +286,19 @@ public class ByteBufList extends AbstractReferenceCounted {
             if (msg instanceof ByteBufList) {
                 ByteBufList b = (ByteBufList) msg;
 
-                ChannelPromise compositePromise = ctx.newPromise();
-                compositePromise.addListener(future -> {
-                    // release the ByteBufList after the write operation is completed
-                    ReferenceCountUtil.safeRelease(b);
-                    // complete the promise passed as an argument unless it's a void promise
-                    if (promise != null && !promise.isVoid()) {
-                        if (future.isSuccess()) {
-                            promise.setSuccess();
-                        } else {
-                            promise.setFailure(future.cause());
-                        }
+                try {
+                    // Write each buffer individually on the socket. The retain() here is needed to preserve the fact
+                    // that ByteBuf are automatically released after a write. If the ByteBufPair ref count is increased
+                    // and it gets written multiple times, the individual buffers refcount should be reflected as well.
+                    int buffersCount = b.buffers.size();
+                    for (int i = 0; i < buffersCount; i++) {
+                        ByteBuf bx = b.buffers.get(i);
+                        // Last buffer will carry on the final promise to notify when everything was written on the
+                        // socket
+                        ctx.write(bx.retainedDuplicate(), i == (buffersCount - 1) ? promise : ctx.voidPromise());
                     }
-                });
-
-                // Write each buffer individually on the socket. The retain() here is needed to preserve the fact
-                // that ByteBuf are automatically released after a write. If the ByteBufPair ref count is increased
-                // and it gets written multiple times, the individual buffers refcount should be reflected as well.
-                int buffersCount = b.buffers.size();
-                for (int i = 0; i < buffersCount; i++) {
-                    ByteBuf bx = b.buffers.get(i);
-                    // Last buffer will carry on the final promise to notify when everything was written on the
-                    // socket
-                    ctx.write(bx.retainedDuplicate(), i == (buffersCount - 1) ? compositePromise : ctx.voidPromise());
+                } finally {
+                    ReferenceCountUtil.release(b);
                 }
             } else {
                 ctx.write(msg, promise);
