@@ -38,6 +38,20 @@ import java.util.concurrent.locks.StampedLock;
  * no node allocations are required to store the keys and values, and no boxing is required.
  *
  * <p>Keys <strong>MUST</strong> be &gt;= 0.
+ * <br>
+ * <b>WARN: method forEach do not guarantee thread safety, nor do the keys, values and asMap method.</b>
+ * <br>
+ * The forEach method is specifically designed for single-threaded usage. When iterating over a map
+ * with concurrent writes, it becomes possible for new values to be either observed or not observed.
+ * There is no guarantee that if we write value1 and value2, and are able to see value2, then we will also see value1.
+ * In some cases, it is even possible to encounter two mappings with the same key,
+ * leading the keys method to return a List containing two identical keys.
+ *
+ * <br>
+ * It is crucial to understand that the results obtained from aggregate status methods such as keys, values, and asMap
+ * are typically reliable only when the map is not undergoing concurrent updates from other threads.
+ * When concurrent updates are involved, the results of these methods reflect transient states
+ * that may be suitable for monitoring or estimation purposes, but not for program control.
  */
 public class ConcurrentLongLongPairHashMap {
 
@@ -279,6 +293,12 @@ public class ConcurrentLongLongPairHashMap {
         }
     }
 
+    /**
+     * Iterate over all the entries in the map and apply the processor function to each of them.
+     * <p>
+     * <b>Warning: Do Not Guarantee Thread-Safety.</b>
+     * @param processor the processor to process the elements.
+     */
     public void forEach(BiConsumerLongPair processor) {
         for (Section s : sections) {
             s.forEach(processor);
@@ -309,9 +329,6 @@ public class ConcurrentLongLongPairHashMap {
     // A section is a portion of the hash map that is covered by a single
     @SuppressWarnings("serial")
     private static final class Section extends StampedLock {
-        // Each item take up 4 continuous array space.
-        private static final int ITEM_SIZE = 4;
-
         // Keys and values are stored interleaved in the table array
         private volatile long[] table;
 
@@ -331,7 +348,7 @@ public class ConcurrentLongLongPairHashMap {
                 float expandFactor, float shrinkFactor) {
             this.capacity = alignToPowerOfTwo(capacity);
             this.initCapacity = this.capacity;
-            this.table = new long[ITEM_SIZE * this.capacity];
+            this.table = new long[4 * this.capacity];
             this.size = 0;
             this.usedBuckets = 0;
             this.autoShrink = autoShrink;
@@ -347,10 +364,7 @@ public class ConcurrentLongLongPairHashMap {
         LongPair get(long key1, long key2, int keyHash) {
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
-            // add local variable here, so OutOfBound won't happen
-            long[] table = this.table;
-            // calculate table.length / 4 as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+            int bucket = signSafeMod(keyHash, capacity);
 
             try {
                 while (true) {
@@ -373,9 +387,8 @@ public class ConcurrentLongLongPairHashMap {
                         if (!acquiredLock) {
                             stamp = readLock();
                             acquiredLock = true;
-                            // update local variable
-                            table = this.table;
-                            bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+
+                            bucket = signSafeMod(keyHash, capacity);
                             storedKey1 = table[bucket];
                             storedKey2 = table[bucket + 1];
                             storedValue1 = table[bucket + 2];
@@ -390,7 +403,7 @@ public class ConcurrentLongLongPairHashMap {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
             } finally {
                 if (acquiredLock) {
@@ -442,7 +455,7 @@ public class ConcurrentLongLongPairHashMap {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
             } finally {
                 if (usedBuckets > resizeThresholdUp) {
@@ -483,7 +496,7 @@ public class ConcurrentLongLongPairHashMap {
                         return false;
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 4) & (table.length - 1);
                 }
 
             } finally {
@@ -509,7 +522,7 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void cleanBucket(int bucket) {
-            int nextInArray = (bucket + ITEM_SIZE) & (table.length - 1);
+            int nextInArray = (bucket + 4) & (table.length - 1);
             if (table[nextInArray] == EmptyKey) {
                 table[bucket] = EmptyKey;
                 table[bucket + 1] = EmptyKey;
@@ -519,7 +532,7 @@ public class ConcurrentLongLongPairHashMap {
 
                 // Cleanup all the buckets that were in `DeletedKey` state,
                 // so that we can reduce unnecessary expansions
-                bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket - 4) & (table.length - 1);
                 while (table[bucket] == DeletedKey) {
                     table[bucket] = EmptyKey;
                     table[bucket + 1] = EmptyKey;
@@ -527,7 +540,7 @@ public class ConcurrentLongLongPairHashMap {
                     table[bucket + 3] = ValueNotFound;
                     --usedBuckets;
 
-                    bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket - 4) & (table.length - 1);
                 }
             } else {
                 table[bucket] = DeletedKey;
@@ -570,7 +583,7 @@ public class ConcurrentLongLongPairHashMap {
                 }
 
                 // Go through all the buckets for this section
-                for (int bucket = 0; bucket < table.length; bucket += ITEM_SIZE) {
+                for (int bucket = 0; bucket < table.length; bucket += 4) {
                     long storedKey1 = table[bucket];
                     long storedKey2 = table[bucket + 1];
                     long storedValue1 = table[bucket + 2];
@@ -599,11 +612,11 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void rehash(int newCapacity) {
-            long[] newTable = new long[ITEM_SIZE * newCapacity];
+            long[] newTable = new long[4 * newCapacity];
             Arrays.fill(newTable, EmptyKey);
 
             // Re-hash table
-            for (int i = 0; i < table.length; i += ITEM_SIZE) {
+            for (int i = 0; i < table.length; i += 4) {
                 long storedKey1 = table[i];
                 long storedKey2 = table[i + 1];
                 long storedValue1 = table[i + 2];
@@ -623,7 +636,7 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private void shrinkToInitCapacity() {
-            long[] newTable = new long[ITEM_SIZE * initCapacity];
+            long[] newTable = new long[4 * initCapacity];
             Arrays.fill(newTable, EmptyKey);
 
             table = newTable;
@@ -637,7 +650,7 @@ public class ConcurrentLongLongPairHashMap {
         }
 
         private static void insertKeyValueNoLock(long[] table, int capacity, long key1, long key2, long value1,
-                                                 long value2) {
+                long value2) {
             int bucket = signSafeMod(hash(key1, key2), capacity);
 
             while (true) {
@@ -652,7 +665,7 @@ public class ConcurrentLongLongPairHashMap {
                     return;
                 }
 
-                bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket + 4) & (table.length - 1);
             }
         }
     }
@@ -671,8 +684,6 @@ public class ConcurrentLongLongPairHashMap {
     }
 
     static final int signSafeMod(long n, int max) {
-        // as the ITEM_SIZE of Section is 4, so the index is the multiple of 4
-        // that is to left shift 2 bits
         return (int) (n & (max - 1)) << 2;
     }
 
