@@ -36,6 +36,20 @@ import java.util.function.Function;
  * <p>Provides similar methods as a {@code ConcurrentMap<K,V>} but since it's an open hash map with linear probing,
  * no node allocations are required to store the values
  *
+ * <br>
+ * <b>WARN: method forEach do not guarantee thread safety, nor do the keys and values method.</b>
+ * <br>
+ * The forEach method is specifically designed for single-threaded usage. When iterating over a map
+ * with concurrent writes, it becomes possible for new values to be either observed or not observed.
+ * There is no guarantee that if we write value1 and value2, and are able to see value2, then we will also see value1.
+ * In some cases, it is even possible to encounter two mappings with the same key,
+ * leading the keys method to return a List containing two identical keys.
+ *
+ * <br>
+ * It is crucial to understand that the results obtained from aggregate status methods such as keys and values
+ * are typically reliable only when the map is not undergoing concurrent updates from other threads.
+ * When concurrent updates are involved, the results of these methods reflect transient states
+ * that may be suitable for monitoring or estimation purposes, but not for program control.
  * @param <V>
  */
 @SuppressWarnings("unchecked")
@@ -243,6 +257,12 @@ public class ConcurrentOpenHashMap<K, V> {
         }
     }
 
+    /**
+     * Iterate over all the entries in the map and apply the processor function to each of them.
+     * <p>
+     * <b>Warning: Do Not Guarantee Thread-Safety.</b>
+     * @param processor the function to apply to each entry
+     */
     public void forEach(BiConsumer<? super K, ? super V> processor) {
         for (Section<K, V> s : sections) {
             s.forEach(processor);
@@ -278,9 +298,6 @@ public class ConcurrentOpenHashMap<K, V> {
     // A section is a portion of the hash map that is covered by a single
     @SuppressWarnings("serial")
     private static final class Section<K, V> extends StampedLock {
-        // Each item take up 2 continuous array space.
-        private static final int ITEM_SIZE = 2;
-
         // Keys and values are stored interleaved in the table array
         private volatile Object[] table;
 
@@ -300,7 +317,7 @@ public class ConcurrentOpenHashMap<K, V> {
                 float expandFactor, float shrinkFactor) {
             this.capacity = alignToPowerOfTwo(capacity);
             this.initCapacity = this.capacity;
-            this.table = new Object[ITEM_SIZE * this.capacity];
+            this.table = new Object[2 * this.capacity];
             this.size = 0;
             this.usedBuckets = 0;
             this.autoShrink = autoShrink;
@@ -315,11 +332,7 @@ public class ConcurrentOpenHashMap<K, V> {
         V get(K key, int keyHash) {
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
-
-            // add local variable here, so OutOfBound won't happen
-            Object[] table = this.table;
-            // calculate table.length / 2 as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+            int bucket = signSafeMod(keyHash, capacity);
 
             try {
                 while (true) {
@@ -341,9 +354,7 @@ public class ConcurrentOpenHashMap<K, V> {
                             stamp = readLock();
                             acquiredLock = true;
 
-                            // update local variable
-                            table = this.table;
-                            bucket = signSafeMod(keyHash, table.length / ITEM_SIZE);
+                            bucket = signSafeMod(keyHash, capacity);
                             storedKey = (K) table[bucket];
                             storedValue = (V) table[bucket + 1];
                         }
@@ -356,7 +367,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
             } finally {
                 if (acquiredLock) {
@@ -409,7 +420,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         }
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
             } finally {
                 if (usedBuckets > resizeThresholdUp) {
@@ -447,7 +458,7 @@ public class ConcurrentOpenHashMap<K, V> {
                         return null;
                     }
 
-                    bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket + 2) & (table.length - 1);
                 }
 
             } finally {
@@ -505,7 +516,7 @@ public class ConcurrentOpenHashMap<K, V> {
                 }
 
                 // Go through all the buckets for this section
-                for (int bucket = 0; bucket < table.length; bucket += ITEM_SIZE) {
+                for (int bucket = 0; bucket < table.length; bucket += 2) {
                     K storedKey = (K) table[bucket];
                     V storedValue = (V) table[bucket + 1];
 
@@ -535,7 +546,7 @@ public class ConcurrentOpenHashMap<K, V> {
             int removedCount = 0;
             try {
                 // Go through all the buckets for this section
-                for (int bucket = 0; size > 0 && bucket < table.length; bucket += ITEM_SIZE) {
+                for (int bucket = 0; size > 0 && bucket < table.length; bucket += 2) {
                     K storedKey = (K) table[bucket];
                     V storedValue = (V) table[bucket + 1];
 
@@ -573,7 +584,7 @@ public class ConcurrentOpenHashMap<K, V> {
         }
 
         private void cleanBucket(int bucket) {
-            int nextInArray = (bucket + ITEM_SIZE) & (table.length - 1);
+            int nextInArray = (bucket + 2) & (table.length - 1);
             if (table[nextInArray] == EmptyKey) {
                 table[bucket] = EmptyKey;
                 table[bucket + 1] = null;
@@ -581,13 +592,13 @@ public class ConcurrentOpenHashMap<K, V> {
 
                 // Cleanup all the buckets that were in `DeletedKey` state,
                 // so that we can reduce unnecessary expansions
-                bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket - 2) & (table.length - 1);
                 while (table[bucket] == DeletedKey) {
                     table[bucket] = EmptyKey;
                     table[bucket + 1] = null;
                     --usedBuckets;
 
-                    bucket = (bucket - ITEM_SIZE) & (table.length - 1);
+                    bucket = (bucket - 2) & (table.length - 1);
                 }
             } else {
                 table[bucket] = DeletedKey;
@@ -597,10 +608,10 @@ public class ConcurrentOpenHashMap<K, V> {
 
         private void rehash(int newCapacity) {
             // Expand the hashmap
-            Object[] newTable = new Object[ITEM_SIZE * newCapacity];
+            Object[] newTable = new Object[2 * newCapacity];
 
             // Re-hash table
-            for (int i = 0; i < table.length; i += ITEM_SIZE) {
+            for (int i = 0; i < table.length; i += 2) {
                 K storedKey = (K) table[i];
                 V storedValue = (V) table[i + 1];
                 if (storedKey != EmptyKey && storedKey != DeletedKey) {
@@ -618,7 +629,7 @@ public class ConcurrentOpenHashMap<K, V> {
         }
 
         private void shrinkToInitCapacity() {
-            Object[] newTable = new Object[ITEM_SIZE * initCapacity];
+            Object[] newTable = new Object[2 * initCapacity];
 
             table = newTable;
             size = 0;
@@ -643,7 +654,7 @@ public class ConcurrentOpenHashMap<K, V> {
                     return;
                 }
 
-                bucket = (bucket + ITEM_SIZE) & (table.length - 1);
+                bucket = (bucket + 2) & (table.length - 1);
             }
         }
     }
@@ -659,8 +670,6 @@ public class ConcurrentOpenHashMap<K, V> {
     }
 
     static final int signSafeMod(long n, int max) {
-        // as the ITEM_SIZE of Section is 2, so the index is the multiple of 2
-        // that is to left shift 1 bit
         return (int) (n & (max - 1)) << 1;
     }
 
