@@ -35,6 +35,18 @@ import java.util.function.Consumer;
  * <p>Provides similar methods as a {@code ConcurrentMap<K,V>} but since it's an open hash map with linear probing,
  * no node allocations are required to store the values
  *
+ * <br>
+ * <b>WARN: method forEach do not guarantee thread safety, nor does the values method.</b>
+ * <br>
+ * The forEach method is specifically designed for single-threaded usage. When iterating over a set
+ * with concurrent writes, it becomes possible for new values to be either observed or not observed.
+ * There is no guarantee that if we write value1 and value2, and are able to see value2, then we will also see value1.
+ *
+ * <br>
+ * It is crucial to understand that the results obtained from aggregate status methods such as values
+ * are typically reliable only when the map is not undergoing concurrent updates from other threads.
+ * When concurrent updates are involved, the results of these methods reflect transient states
+ * that may be suitable for monitoring or estimation purposes, but not for program control.
  * @param <V>
  */
 @SuppressWarnings("unchecked")
@@ -216,6 +228,12 @@ public class ConcurrentOpenHashSet<V> {
         }
     }
 
+    /**
+     * Iterate over all the elements in the set and apply the provided function.
+     * <p>
+     * <b>Warning: Do Not Guarantee Thread-Safety.</b>
+     * @param processor the function to apply to each element
+     */
     public void forEach(Consumer<? super V> processor) {
         for (Section<V> s : sections) {
             s.forEach(processor);
@@ -265,16 +283,16 @@ public class ConcurrentOpenHashSet<V> {
         }
 
         boolean contains(V value, int keyHash) {
+            int bucket = keyHash;
+
             long stamp = tryOptimisticRead();
             boolean acquiredLock = false;
 
-            // add local variable here, so OutOfBound won't happen
-            V[] values = this.values;
-            // calculate table.length as capacity to avoid rehash changing capacity
-            int bucket = signSafeMod(keyHash, values.length);
-
             try {
                 while (true) {
+                    int capacity = this.capacity;
+                    bucket = signSafeMod(bucket, capacity);
+
                     // First try optimistic locking
                     V storedValue = values[bucket];
 
@@ -292,12 +310,15 @@ public class ConcurrentOpenHashSet<V> {
                             stamp = readLock();
                             acquiredLock = true;
 
-                            // update local variable
-                            values = this.values;
-                            bucket = signSafeMod(keyHash, values.length);
                             storedValue = values[bucket];
                         }
 
+                        if (capacity != this.capacity) {
+                            // There has been a rehashing. We need to restart the search
+                            bucket = keyHash;
+                            continue;
+                        }
+
                         if (value.equals(storedValue)) {
                             return true;
                         } else if (storedValue == EmptyValue) {
@@ -305,7 +326,8 @@ public class ConcurrentOpenHashSet<V> {
                             return false;
                         }
                     }
-                    bucket = (bucket + 1) & (values.length - 1);
+
+                    ++bucket;
                 }
             } finally {
                 if (acquiredLock) {
