@@ -19,8 +19,6 @@
 package org.apache.bookkeeper.conf;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.CALLS_REAL_METHODS;
 import static org.mockito.Mockito.mock;
 
@@ -181,12 +179,4 @@ public class AbstractConfigurationTest {
             System.getProperties().put(nettyLevelKey, nettyLevelStr);
         }
     }
-
-    @Test
-    public void testExitOnOutOfMemory() {
-        assertFalse(conf.exitOnOutOfMemory());
-        conf.setExitOnOutOfMemory(true);
-        assertTrue(conf.exitOnOutOfMemory());
-    }
-
 }
