@@ -23,8 +23,8 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.mockito.Mockito.spy;
 import static org.mockito.Mockito.when;
-import static org.powermock.api.mockito.PowerMockito.spy;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
@@ -1023,43 +1023,11 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         assertFalse(readOnlyState.isReadOnly());
 
         //forceReadonly to writable
-        MetadataBookieDriver metadataDriver = BookieResources.createMetadataDriver(
-                baseConf, NullStatsLogger.INSTANCE);
-        restartBookies(c -> {
-            c.setForceReadOnlyBookie(true);
-            c.setReadOnlyModeEnabled(true);
-            return c;
-        });
-        // the old bkHttpServiceProvider has an old bookie instance who has been shutdown
-        // so we need create a new bkHttpServiceProvider2 to contains a new bookie which has created by restart.
-        BKHttpServiceProvider bkHttpServiceProvider2 = new BKHttpServiceProvider.Builder()
-                .setBookieServer(serverByIndex(numberOfBookies - 1))
-                .setServerConfiguration(baseConf)
-                .setLedgerManagerFactory(metadataDriver.getLedgerManagerFactory())
-                .build();
-        HttpEndpointService bookieReadOnlyService2 = bkHttpServiceProvider2
-                .provideHttpEndpointService(HttpServer.ApiType.BOOKIE_STATE_READONLY);
-
+        baseConf.setForceReadOnlyBookie(true);
+        baseConf.setReadOnlyModeEnabled(true);
+        restartBookies();
         request = new HttpServiceRequest(JsonUtil.toJson(new ReadOnlyState(false)), HttpServer.Method.PUT,  null);
-        response = bookieReadOnlyService2.handle(request);
-        assertEquals(400, response.getStatusCode());
-
-        // disable readOnly mode
-        restartBookies(c -> {
-            c.setForceReadOnlyBookie(false);
-            c.setReadOnlyModeEnabled(false);
-            return c;
-        });
-        bkHttpServiceProvider2 = new BKHttpServiceProvider.Builder()
-                .setBookieServer(serverByIndex(numberOfBookies - 1))
-                .setServerConfiguration(baseConf)
-                .setLedgerManagerFactory(metadataDriver.getLedgerManagerFactory())
-                .build();
-        bookieReadOnlyService2 = bkHttpServiceProvider2
-                .provideHttpEndpointService(HttpServer.ApiType.BOOKIE_STATE_READONLY);
-
-        request = new HttpServiceRequest(JsonUtil.toJson(new ReadOnlyState(true)), HttpServer.Method.PUT,  null);
-        response = bookieReadOnlyService2.handle(request);
+        response = bookieReadOnlyService.handle(request);
         assertEquals(400, response.getStatusCode());
     }
 
