@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -24,6 +24,7 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.junit.Assume.assumeTrue;
+
 import java.io.File;
 import java.io.IOException;
 import java.security.cert.Certificate;
@@ -127,6 +128,7 @@ public class TestTLS extends BookKeeperClusterTestCase {
         baseClientConf.setTLSProviderFactoryClass(TLSContextFactory.class.getName());
         baseClientConf.setTLSClientAuthentication(true);
         baseClientConf.setUseV2WireProtocol(useV2Protocol);
+        baseClientConf.setLimitStatsLogging(false);
 
         switch (clientKeyStoreFormat) {
         case PEM:
@@ -903,42 +905,42 @@ public class TestTLS extends BookKeeperClusterTestCase {
                     .append(TestUtils.buildStatsCounterPathFromBookieID(bookie.getBookieId()))
                     .append(".");
             // check stats on TLS enabled client
-           TestStatsProvider.TestCounter cntr =  tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+           TestStatsProvider.TestCounter cntr =  tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_TLS_CHANNEL_COUNTER);
 
             assertEquals("Mismatch TLS channel count", 1,
-                    tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_TLS_CHANNEL_COUNTER).get().longValue());
             assertEquals("TLS handshake failure unexpected", 0,
-                    tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.FAILED_TLS_HANDSHAKE_COUNTER).get().longValue());
             assertEquals("Mismatch non-TLS channel count", 0,
-                    tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_NON_TLS_CHANNEL_COUNTER).get().longValue());
             assertEquals("Connection failures unexpected", 0,
-                    tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.FAILED_CONNECTION_COUNTER).get().longValue());
 
             // check stats on non-TLS enabled client
             assertEquals("Mismatch TLS channel count", 0,
-                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_TLS_CHANNEL_COUNTER).get().longValue());
             assertEquals("TLS handshake failure unexpected", 0,
-                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.FAILED_TLS_HANDSHAKE_COUNTER).get().longValue());
             assertEquals("Mismatch non-TLS channel count", 1,
-                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_NON_TLS_CHANNEL_COUNTER).get().longValue());
             assertEquals("Connection failures unexpected", 0,
-                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.FAILED_CONNECTION_COUNTER).get().longValue());
 
             bookie.shutdown();
             assertEquals("Mismatch TLS channel count", 0,
-                    tlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    tlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_TLS_CHANNEL_COUNTER).get().longValue());
             assertEquals("Mismatch non-TLS channel count", 0,
-                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder.toString()
+                    nonTlsClient.getTestStatsProvider().getCounter(nameBuilder
                     + BookKeeperClientStats.ACTIVE_NON_TLS_CHANNEL_COUNTER).get().longValue());
 
         }
@@ -1001,7 +1003,7 @@ public class TestTLS extends BookKeeperClusterTestCase {
                 .append(TestUtils.buildStatsCounterPathFromBookieID(bookie.getBookieId()))
                 .append(".");
         assertEquals("TLS handshake failure expected", 1,
-                client.getTestStatsProvider().getCounter(nameBuilder.toString()
+                client.getTestStatsProvider().getCounter(nameBuilder
                 + BookKeeperClientStats.FAILED_TLS_HANDSHAKE_COUNTER).get().longValue());
     }
 
